/*
 * This file defines the SIP library internal interfaces.
 *
 * Terms and Conditions
 * 
 * Copyright (c) 1998, 1999, 2000, 2001
 * 	Phil Thompson <phil@river-bank.demon.co.uk>
 * Copyright (c) 2002
 * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
 * IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * Except as contained in this notice, the name of the copyright holder shall
 * not be used in advertising or otherwise to promote the sale, use or other
 * dealings in this Software without prior written authorization from the
 * copyright holder.
 * 
 * These conditions apply only to the Software, no conditions are placed on
 * program code or documentation generated by the Software.
 */


#ifndef _SIPINT_H
#define	_SIPINT_H


#ifdef __cplusplus
extern "C" {
#endif

#undef	TRUE
#define	TRUE		1

#undef	FALSE
#define	FALSE		0


/*
 * This defines an entry in a linked list of Python objects.
 */

typedef struct _sipThisList
{
	sipThisType *sipThis;		/* This Python object. */
	struct _sipThisList *next;	/* The next in the list. */
} sipThisList;


/*
 * This defines a single entry in an object map's hash table.
 */

typedef struct
{
	const void *key;		/* The C/C++ address. */
	sipThisList *vallist;		/* The list of objects at this address. */
	sipThisList first;		/* Storage for the first in the list. */
} sipHashEntry;


/*
 * This defines the interface to a hash table class for mapping C/C++ addresses
 * to the corresponding wrapped Python object.
 */

typedef struct
{
	int primeIdx;			/* Index into table sizes. */
	unsigned long size;		/* Size of hash table. */
	unsigned long neverUsed;	/* Nr. never used in hash table. */
	sipHashEntry *hash_array;	/* Current hash table. */
} sipObjectMap;


extern PyThreadState *sipMainThreadState;	/* The main thread state. */


#ifdef SIP_QT_SUPPORT
extern PyObject *sipQObjectClass;	/* The Python QObject class. */

const void *sipConvertRx Py_PROTO((void *,sipThisType *,char *,PyObject *,
				   char *,char **));
const void *sipGetRx Py_PROTO((sipThisType *,char *,PyObject *,char *,char **));
#endif


int sipFindPyMethod Py_PROTO((sipMethodCache *,sipThisType *,char *,char *));

void sipOMInit Py_PROTO((sipObjectMap *));
void sipOMFinalise Py_PROTO((sipObjectMap *));
sipThisType *sipOMFindObject Py_PROTO((sipObjectMap *,const void *,PyObject *));
void sipOMAddObject Py_PROTO((sipObjectMap *,const void *,sipThisType *));
int sipOMRemoveObject Py_PROTO((sipObjectMap *,const void *,sipThisType *));


#ifdef __cplusplus
}
#endif

#endif
